// the site for this project is here:
// http://github.com/jonkruger/jsview


// thanks to http://yelotofu.com/2008/08/jquery-outerhtml/ for this function
jQuery.fn.outerHTML = function(s)
{
    return (s)
        ? this.before(s).remove()
        : jQuery("<p>").append(this.eq(0).clone()).html();
}

String.prototype.startsWith = function(str)
{
    return (this.match("^" + str) == str)
}

function jQueryView(objectName, jQueryElement)
{
    var propertyRegistry = new PropertyRegistry();
    var getTextFunctions = new Array();
    var getValueFunctions = new Array();
    var setValueFunctions = new Array();
    var propertyChangeCallbacks = new Array();
    var theObj = null;

    if(jQueryElement.html() == undefined ||
        jQueryElement.html() == null ||
        jQueryElement.html().length == 0)
    {
        throw ('Element for object ' + objectName + ' has no HTML.');
    }

    if(jQueryElement.length > 1)
        throw ('jQueryElement for object ' + objectName + ' represents multiple elements.  Make sure that the query that you are using to get the element retrieves only one element.');

    this.isTestDouble = function()
    {
        return false;
    }

    this.hasEvents = function()
    {
        return true;
    }

    this.registerProperty = function(propertyName, getValueFunction, setValueFunction)
    {
        propertyRegistry.registerProperty(propertyName);
        if(getValueFunction != undefined && getValueFunction != null)
            getValueFunctions[propertyName] = getValueFunction;
        if(setValueFunction != undefined && setValueFunction != null)
            setValueFunctions[propertyName] = setValueFunction;
    }

    this.unregisterProperty = function(propertyName)
    {
        propertyRegistry.unregisterProperty(propertyName);
    }

    this.registerList = function(listName)
    {
        // nothing to do here (yet)
    }

    this.unregisterPropertiesWhere = function(criteria)
    {
        propertyRegistry.unregisterPropertiesWhere(criteria);
    }

    this.isPropertyRegistered = function(propertyName)
    {
        return propertyRegistry.isPropertyRegistered(propertyName);
    }

    this.getObjectName = function()
    {
        return objectName;
    }

    this.getValue = function(propertyName)
    {
        var getValueFunction = getValueFunctions[propertyName];
        if(getValueFunction != undefined)
            getValueFunction();
        else
        {
            var element = this.getElement(propertyName);
            if(element.length == 0)
            {
                throw ('No element was found for property ' + propertyName + '.');
                return;
            }

            if(element.is('input[type="checkbox"]') || element.is('input[type="radio"]'))
                return element.is(':checked');
            else if(element.is('input') || element.is('textarea'))
                return element.val();
            else if(element.is('select') && element.attr('multiple'))
            {
                var result = new Array();
                element.find('option:selected').each(function() { result.push($(this).val()); });
                return result;
            }
            else if(element.is('select'))
                return element.find('option:selected').val();
            return element.text();
        }
    }

    this.setValue = function(propertyName, value)
    {
        var setValueFunction = setValueFunctions[propertyName];
        if (setValueFunction != undefined)
            setValueFunction(value);
        else
        {
            var element = this.getElement(propertyName);
            if (element.length == 0)
            {
                throw ('No element was found for property ' + propertyName + '.');
                return;
            }

            // This is here because IE will actually put word "null" in the text/value
            // attributes of controls.  
            if (value == null)
                value = '';
            
            if (element.is('input[type="checkbox"]') || element.is('input[type="radio"]'))
                element.attr('checked', value);
            else if (element.is('input') || element.is('textarea'))
                element.val(value);
            else if (element.is('select') && element.attr('multiple'))
            {
                // deselect everything
                element.find('option:selected').each(function() { $(this).attr('selected', false); });

                if (isArray(value))
                {
                    for (var i = 0; i < value.length; i++)
                        element.find('option[value="' + value[i] + '"]').attr('selected', true);
                }
                else
                    element.find('option[value="' + value + '"]').attr('selected', true);
            }
            else if (element.is('select'))
                return element.find('option[value="' + value + '"]').attr('selected', 'selected');
            else
                element.text(value);
        }

        if (propertyChangeCallbacks[propertyName] != null)
        {
            var callbackArray = propertyChangeCallbacks[propertyName];
            for (var i = 0; i < callbackArray.length; i++)
            {
                callbackArray[i].call(theObj);
            }
        }
    }

    this.getText = function(propertyName)
    {
        var element = this.getElement(propertyName);
        if(element.is('input') || element.is('textarea'))
            return element.val();
        else if(element.is('select'))
            return element.find('option:selected').text();
        return element.text();
    }

    this.outerHTML = function()
    {
        return jQueryElement.outerHTML();
    }

    this.getObjects = function(objectType, creationFunction)
    {
        var result = new Array();
        var objects = jQueryElement.find('[class*="' + objectType + '-Object"]');
        jQuery.each(objects, function()
        {
            var object = creationFunction($(this));
            result.push(object);
        });
        return result;
    }

    this.numberOfObjects = function(objectType)
    {
        return jQueryElement.find('[class*="' + objectType + '-Object"]').length;
    }

    this.showElement = function(propertyName)
    {
        var element = this.getElement(propertyName);
        element.show();
    }

    this.hideElement = function(propertyName)
    {
        var element = this.getElement(propertyName);
        element.hide();
    }

    this.show = function()
    {
        jQueryElement.show();
    }

    this.hide = function()
    {
        jQueryElement.hide();
    }

    this.hasElement = function()
    {
        return true;
    }

    this.getElement = function(propertyName)
    {
        if(propertyName == undefined)
            return jQueryElement;
        return jQueryElement.find('.' + objectName + '-' + propertyName);
    }

    this.getElementId = function(propertyName)
    {
        var element = this.getElement(propertyName);
        return element.attr('id');
    }

    this.setElementId = function(propertyName, id)
    {
        var element = this.getElement(propertyName);
        element.attr('id', id);
    }

    this.getElementName = function(propertyName)
    {
        var element = this.getElement(propertyName);
        return element.attr('name');
    }

    this.setElementName = function(propertyName, name)
    {
        var element = this.getElement(propertyName);
        element.attr('name', name);
    }

    this.shouldRegisterClickEventInsteadOfChange = function(propertyName)
    {
        return this.isCheckBox(propertyName);
    }

    this.isCheckBox = function(propertyName)
    {
        return this.getElement(propertyName).is('input:checkbox');
    }

    this.isChecked = function(propertyName)
    {
        var element = this.getElement(propertyName);
        return element.attr('checked');
    }

    this.disableElement = function(propertyName)
    {
        this.getElement(propertyName).attr('disabled', 'disabled');
    }

    this.enableElement = function(propertyName)
    {
        this.getElement(propertyName).attr('disabled', '');
    }

    this.blur = function(propertyName)
    {
        this.getElement(propertyName).blur();
    }

    this.addClass = function(propertyName, cssClass)
    {
        this.getElement(propertyName).addClass(cssClass);
    }

    this.removeClass = function(propertyName, cssClass)
    {
        this.getElement(propertyName).removeClass(cssClass);
    }

    this.registerEventHandler = function(obj, propertyName, event, arg1, arg2, arg3)
    {
        theObj = obj;

        this.getElement(propertyName)[event](function(event) { arg1.call(theObj, event) }, arg2, arg3);

        if(event == 'change')
        {
            var callbackArray = null;
            if(propertyChangeCallbacks[propertyName] == null)
            {
                callbackArray = new Array();
                propertyChangeCallbacks[propertyName] = callbackArray;
            }
            else
                callbackArray = propertyChangeCallbacks[propertyName];
            callbackArray.push(arg1);
        }
    }

    this.appendListItem = function(obj, listName, item)
    {
      $.tmpl(listName, [item.toJson()]).appendTo(this.getElement(listName));
    }

    this.prependListItem = function(obj, listName, item)
    {
      $.tmpl(listName, [item.toJson()]).prependTo(this.getElement(listName));
    }

    this.getAttribute = function(propertyName, attributeName)
    {
        return this.getElement(propertyName).attr(attributeName);
    }

    this.setAttribute = function(propertyName, attributeName, value)
    {
        return this.getElement(propertyName).attr(attributeName, value);
    }

    this.buildObjectFromValues = function(propertyNameArray)
    {
        var object = {};
        for(var i = 0; i < propertyNameArray.length; i++)
        {
            var propertyName = propertyNameArray[i];

            // We have to check for this because people can extend the Array
            // class, which will add functions to the list of properties on
            // the Array object passed in.
            if(typeof (propertyName) == 'string')
                object[propertyName] = this.getValue(propertyName);
        }
        return object;
    }

    this.buildObjectFromRegisteredPropertyValues = function()
    {
        return this.buildObjectFromValues(propertyRegistry.getAllProperties());
    }

    this.toJson = function()
    {
      var json = {};
      for(var i = 0; i < propertyNameArray.length; i++)
      {
        json[propertyNameArray[i]] = this.getValue(propertyNameArray[i]);
      }
      return json;
    }
}

function PropertyRegistry()
{
    var properties = new Array();

    this.getAllProperties = function()
    {
        return properties;
    }

    this.registerProperty = function(propertyName, getValueFunction, setValueFunction)
    {
        properties.push(propertyName);
    }

    this.unregisterProperty = function(propertyName)
    {
        for(var i = properties.length - 1; i >= 0; i--)
        {
            if(properties[i] == propertyName)
            {
                properties.splice(i, 1);
                return;
            }
        }
    }

    this.unregisterPropertiesWhere = function(criteria)
    {
        for(var i = properties.length - 1; i >= 0; i--)
        {
            if(criteria(properties[i]) == true)
                properties.splice(i, 1);
        }
    }

    this.isPropertyRegistered = function(propertyName)
    {
        for(var i = 0; i < properties.length; i++)
        {
            if(properties[i] == propertyName)
                return true;
        }
        return false;
    }
}

function UnboundView()
{
    var propertyRegistry = new PropertyRegistry();
    var values = new Array();

    this.registerProperty = function(propertyName, getValueFunction, setValueFunction)
    {
        propertyRegistry.registerProperty(propertyName);
        values[propertyName] = null;
    }

    this.getValue = function(propertyName)
    {
        return values[propertyName];
    }

    this.setValue = function(propertyName, value)
    {
        values[propertyName] = value;
    }

    this.hasElement = function()
    {
        return false;
    }

    this.shouldRegisterClickEventInsteadOfChange = function(propertyName)
    {
        return false;
    }

    this.hasEvents = function()
    {
        return false;
    }

    this.toJson = function()
    {
      var json = {};
      properties = propertyRegistry.getAllProperties();
      for(var i = 0; i < properties.length; i++)
      {
        json[properties[i]] = this.getValue(properties[i]);
      }
      return json;
    }
}

function registerList(obj, view, listName, listItemType)
{
    if (obj.autoGenerationInfo == null || obj.autoGenerationInfo == undefined)
        obj.autoGenerationInfo = new Object();
    var autoGeneratedInfo = obj['autoGenerationInfo'];

  var wasAppendToAutoGeneratedForObject = autoGeneratedInfo['auto-generated-appendTo'];
  var wasPrependToAutoGeneratedForObject = autoGeneratedInfo['auto-generated-prependTo'];
  var wasGetListToAutoGeneratedForObject = autoGeneratedInfo['auto-generated-getList'];

  if (obj['appendTo' + listName] == undefined || wasAppendToAutoGeneratedForObject == true)
  {
    obj['appendTo' + listName] = function(item)
    {
      view.appendListItem(obj, listName, item);
    };
    autoGeneratedInfo['auto-generated-appendTo'] = true;
  }

  if (obj['prependTo' + listName] == undefined || wasPrependToAutoGeneratedForObject == true)
  {
    obj['prependTo' + listName] = function(item)
    {
      view.prependListItem(obj, listName, item);
    };
    autoGeneratedInfo['auto-generated-prependTo'] = true;
  }

  if (obj['get' + listName] == undefined || wasGetListAutoGeneratedForObject == true)
  {
    obj['get' + listName] = function(item)
    {
      return view.getObjects(listName, function(element) { return new listItemType(element); });
    };
    autoGeneratedInfo['auto-generated-getList'] = true;
  }

  view.registerList(listName);
}

function registerObjectProperties(obj, view, arrayOfProperties)
{
    if (obj.autoGenerationInfo == null || obj.autoGenerationInfo == undefined)
        obj.autoGenerationInfo = new Object();
    var autoGeneratedInfo = obj['autoGenerationInfo'];

    var wasWhenClickedAutoGeneratedForObject = autoGeneratedInfo['auto-generated-whenClicked'];
    var wasClickAutoGeneratedForObject = autoGeneratedInfo['auto-generated-click'];
    var wasToJsonAutoGeneratedForObject = autoGeneratedInfo['auto-generated-toJson'];

    if (obj['whenClicked'] == undefined || wasWhenClickedAutoGeneratedForObject == true)
    {
        obj['whenClicked'] = function(callback)
        {
            view.registerEventHandler(obj, null, 'click', callback);
        };
        autoGeneratedInfo['auto-generated-whenClicked'] = true;
    }

    if (obj['click'] == undefined || wasClickAutoGeneratedForObject == true)
    {
        obj['click'] = function(eventObject)
        {
            view.click(null, eventObject);
        };
        autoGeneratedInfo['auto-generated-click'] = true;
    }

    if (obj['toJson'] == undefined || wasToJsonAutoGeneratedForObject == true)
    {
        obj['toJson'] = function()
        {
            return view.toJson();
        };
        autoGeneratedInfo['auto-generated-toJson'] = true;
    }

    // explanation of the funky syntax:
    // http://stackoverflow.com/questions/442985/how-can-one-de-reference-javascript-variables-when-enclosing-an-outer-scope
    for (var i = 0; i < arrayOfProperties.length; i++) (function(i)
    {
        var property = arrayOfProperties[i];

        var wasGetterAutoGenerated = autoGeneratedInfo['auto-generated-get' + property];
        var wasSetterAutoGenerated = autoGeneratedInfo['auto-generated-set' + property];
        var wasWhenChangesAutoGenerated = autoGeneratedInfo['auto-generated-when' + property + 'Changes'];
        var wasWhenIsClickedAutoGenerated = autoGeneratedInfo['auto-generated-when' + property + 'IsClicked'];
        var wasWhenClickedAutoGenerated = autoGeneratedInfo['auto-generated-when' + property + 'Clicked'];
        var wasWhenLosesFocusAutoGenerated = autoGeneratedInfo['auto-generated-when' + property + 'LosesFocus'];
        var wasWhenGainsFocusAutoGenerated = autoGeneratedInfo['auto-generated-when' + property + 'GainsFocus'];
        var wasWhenKeyPressedAutoGenerated = autoGeneratedInfo['auto-generated-whenKeyIsPressedIn' + property];
        var wasWhenKeyDownAutoGenerated = autoGeneratedInfo['auto-generated-whenKeyDownIn' + property];
        var wasShowAutoGenerated = autoGeneratedInfo['auto-generated-show' + property];
        var wasHideAutoGenerated = autoGeneratedInfo['auto-generated-hide' + property];
        var wasClickAutoGenerated = autoGeneratedInfo['auto-generated-click' + property];
        var wasPressKeyAutoGenerated = autoGeneratedInfo['auto-generated-pressKeyIn' + property];
        var wasKeyDownAutoGenerated = autoGeneratedInfo['auto-generated-keyDownIn' + property];
        var wasDisableAutoGenerated = autoGeneratedInfo['auto-generated-Disable' + property];
        var wasEnableAutoGenerated = autoGeneratedInfo['auto-generated-Enable' + property];

        view.registerProperty(property);

        if (obj['get' + property] == undefined || wasGetterAutoGenerated == true)
        {
            obj['get' + property] = function() { return view.getValue(property); }
            autoGeneratedInfo['auto-generated-get' + property] = true;
        }

        if (obj['set' + property] == undefined || wasSetterAutoGenerated == true)
        {
            obj['set' + property] = function(value) { view.setValue(property, value); }
            autoGeneratedInfo['auto-generated-set' + property] = true;
        }

        if (obj['when' + property + 'Changes'] == undefined || wasWhenChangesAutoGenerated == true)
        {
            obj['when' + property + 'Changes'] = function(callback)
            {
                if (view.shouldRegisterClickEventInsteadOfChange(property) == true)
                    view.registerEventHandler(obj, property, 'click', callback);
                else
                    view.registerEventHandler(obj, property, 'change', callback);
            };
            autoGeneratedInfo['auto-generated-when' + property + 'Changes'] = true;
        }

        if (obj['when' + property + 'Clicked'] == undefined || wasWhenClickedAutoGenerated == true)
        {
            obj['when' + property + 'Clicked'] = function(callback)
            {
                view.registerEventHandler(obj, property, 'click', callback);
            };
            autoGeneratedInfo['auto-generated-when' + property + 'Clicked'] = true;
        }

        if (obj['when' + property + 'IsClicked'] == undefined || wasWhenIsClickedAutoGenerated == true)
        {
            obj['when' + property + 'IsClicked'] = function(callback)
            {
                view.registerEventHandler(obj, property, 'click', callback);
            };
            autoGeneratedInfo['auto-generated-when' + property + 'IsClicked'] = true;
        }

        if (obj['when' + property + 'GainsFocus'] == undefined || wasWhenGainsFocusAutoGenerated == true)
        {
            obj['when' + property + 'GainsFocus'] = function(callback)
            {
                view.registerEventHandler(obj, property, 'focus', callback);
            };
            autoGeneratedInfo['auto-generated-when' + property + 'GainsFocus'] = true;
        }

        if (obj['when' + property + 'LosesFocus'] == undefined || wasWhenLosesFocusAutoGenerated == true)
        {
            obj['when' + property + 'LosesFocus'] = function(callback)
            {
                view.registerEventHandler(obj, property, 'blur', callback);
            };
            obj['fireWhen' + property + 'LosesFocus'] = function()
            {
                view.blur(property); 
            };
            autoGeneratedInfo['auto-generated-when' + property + 'LosesFocus'] = true;
        }

        if (obj['whenKeyIsPressedIn' + property] == undefined || wasWhenKeyPressedAutoGenerated == true)
        {
            obj['whenKeyIsPressedIn' + property] = function(callback)
            {
                view.registerEventHandler(obj, property, 'keypress', callback);
            };
            autoGeneratedInfo['auto-generated-whenKeyIsPressedIn' + property] = true;
        }

        if (obj['whenKeyDownIn' + property] == undefined || wasWhenKeyDownAutoGenerated == true)
        {
            obj['whenKeyDownIn' + property] = function(callback)
            {
                view.registerEventHandler(obj, property, 'keydown', callback);
            };
            autoGeneratedInfo['auto-generated-whenKeyDownIn' + property] = true;
        }

        if (obj['show' + property] == undefined || wasShowAutoGenerated == true)
        {
            obj['show' + property] = function()
            {
                view.showElement(property);
            };
            autoGeneratedInfo['auto-generated-show' + property] = true;
        }

        if (obj['hide' + property] == undefined || wasHideAutoGenerated == true)
        {
            obj['hide' + property] = function()
            {
                view.hideElement(property);
            };
            autoGeneratedInfo['auto-generated-hide' + property] = true;
        }

        if (obj['click' + property] == undefined || wasClickAutoGenerated == true)
        {
            obj['click' + property] = function(eventObject)
            {
                view.click(property, eventObject);
            };
            autoGeneratedInfo['auto-generated-click' + property] = true;
        }

        if (obj['pressKeyIn' + property] == undefined || wasPressKeyAutoGenerated == true)
        {
            obj['pressKeyIn' + property] = function(eventObject)
            {
                view.pressKey(property, eventObject);
            };
            autoGeneratedInfo['auto-generated-pressKeyIn' + property] = true;
        }

        if (obj['keyDownIn' + property] == undefined || wasKeyDownAutoGenerated == true)
        {
            obj['keyDownIn' + property] = function(eventObject)
            {
                view.keyDown(property, eventObject);
            };
            autoGeneratedInfo['auto-generated-keyDownIn' + property] = true;
        }

        if (obj['enable' + property] == undefined || wasKeyDownAutoGenerated == true)
        {
            obj['enable' + property] = function(eventObject)
            {
                view.enableElement(property);
            };
            autoGeneratedInfo['auto-generated-enable' + property] = true;
        }

        if (obj['disable' + property] == undefined || wasKeyDownAutoGenerated == true)
        {
            obj['disable' + property] = function(eventObject)
            {
                view.disableElement(property);
            };
            autoGeneratedInfo['auto-generated-disable' + property] = true;
        }
    })(i);
}

function registerAllUserInputElements(obj, view)
{
    var userInputElements = view.getElement().find('input, select, textarea');
    registerElements(obj, view, userInputElements);
}

function registerElementsMarkedWithClassName(obj, view)
{
    var objectName = view.getObjectName();
    var elementSelector = view.getElement().find('*[class*=' + objectName + '-]');
    var properties = new Array();
    for(var i = 0; i < elementSelector.length; i++)
    {
        var element = elementSelector[i];
        var classes = element.className.split(' ');

        for(var classIndex = 0; classIndex < classes.length; classIndex++)
        {
            var theClass = classes[classIndex];
            if(theClass.startsWith(objectName + '-'))
                properties.push(theClass.substring(objectName.length + 1, 10000));
        }
        properties.push(element.id);
    }

    registerObjectProperties(obj, view, properties);
}

function registerElements(obj, view, elementSelector)
{
    var properties = new Array();
    for(var i = 0; i < elementSelector.length; i++)
    {
        var element = elementSelector[i];
        element.className += ' ' + view.getObjectName() + '-' + element.id;
        properties.push(element.id);
    }

    registerObjectProperties(obj, view, properties);
}

function isArray(obj)
{
    if(obj == null || obj.constructor.toString().indexOf("Array") == -1)
        return false;
    else
        return true;
}

// conventions
// - class name in this format "objectType-propertyName"
// - parent elements for an object will have a class name in this format "objectType-Object-primarykey"
// - some way to know that it's a template that should be hidden (if class = "objectType-Template")?
