function ViewTestDouble()
{
    var propertyRegistry = new PropertyRegistry();
    var values = new Array();
    var visible = true;
    var ids = new Array();
    var names = new Array();
    var attributes = new Array();
    var propertyChangeCallbacks = new Object();
    var propertyClickCallbacks = new Object();
    var keyPressCallbacks = new Object();
    var keyDownCallbacks = new Object();
    var blurCallbacks = new Object();
    var objectClickCallbacks = new Array();
    var theObj = null;
    var lists = new Array();

    this.isTestDouble = function()
    {
        return true;
    }

    this.hasEvents = function()
    {
        return true;
    }

    this.registerProperty = function(propertyName, getValueFunction, setValueFunction)
    {
        propertyRegistry.registerProperty(propertyName);
        values[propertyName] = null;
    }

    this.unregisterProperty = function(propertyName)
    {
        propertyRegistry.unregisterProperty(propertyName);
    }

    this.registerList = function(listName)
    {
        lists[listName] = [];
    }

    this.unregisterPropertiesWhere = function(criteria)
    {
        propertyRegistry.unregisterPropertiesWhere(criteria);
    }

    this.isPropertyRegistered = function(propertyName)
    {
        return propertyRegistry.isPropertyRegistered(propertyName);
    }

    this.getObjectName = function()
    {
        return '';
    }

    this.getValue = function(propertyName)
    {
        return values[propertyName];
    }

    this.setValue = function(propertyName, value)
    {
        values[propertyName] = value;

        if(propertyChangeCallbacks[propertyName] != null)
        {
            var callbackArray = propertyChangeCallbacks[propertyName];
            for(var i = 0; i < callbackArray.length; i++)
            {
                callbackArray[i].call(theObj);
            }
        }
    }

    this.click = function(propertyName, eventObject)
    {
        if (propertyName == null)
        {
            for (var i = 0; i < objectClickCallbacks.length; i++)
            {
                objectClickCallbacks[i].call(theObj, eventObject);
            }
        }
        else
        {
            if (propertyClickCallbacks[propertyName] != null)
            {
                var callbackArray = propertyClickCallbacks[propertyName];
                for (var i = 0; i < callbackArray.length; i++)
                {
                    callbackArray[i].call(theObj, eventObject);
                }
            }
        }
    }

    this.pressKey = function(propertyName, eventObject)
    {
        if (keyPressCallbacks[propertyName] != null)
        {
            var callbackArray = keyPressCallbacks[propertyName];
            for (var i = 0; i < callbackArray.length; i++)
            {
                callbackArray[i].call(theObj, eventObject);
            }
        }
    }

    this.keyDown = function(propertyName, eventObject)
    {
        if (keyDownCallbacks[propertyName] != null)
        {
            var callbackArray = keyDownCallbacks[propertyName];
            for (var i = 0; i < callbackArray.length; i++)
            {
                callbackArray[i].call(theObj, eventObject);
            }
        }
    }

    this.blur = function(propertyName)
    {
        if (blurCallbacks[propertyName] != null)
        {
            var callbackArray = blurCallbacks[propertyName];
            for (var i = 0; i < callbackArray.length; i++)
            {
                callbackArray[i].call(theObj);
            }
        }
    }

    this.outerHTML = function()
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
        return '';
    }

    this.getObjects = function(objectType, creationFunction)
    {
        return lists[objectType];
    }

    this.numberOfObjects = function(objectType)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.showElement = function(elementName)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.hideElement = function(elementName)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.show = function()
    {
        visible = true;
    }

    this.hide = function()
    {
        visible = false;
    }

    this.hasElement = function()
    {
        // For this to be a true view test double, this has to return true
        // so that code that calls this method will continue to function
        // as if there is an element behind it.
        return true;
    }

    this.getElement = function(propertyName)
    {
        throw ('You must override the getElement function if it is used.');
    }

    this.getElementId = function(propertyName)
    {
        return ids[propertyName];
    }

    this.setElementId = function(propertyName, id)
    {
        ids[propertyName] = id;
    }

    this.getElementName = function(propertyName)
    {
        return names[propertyName];
    }

    this.setElementName = function(propertyName, name)
    {
        names[propertyName] = name;
    }

    this.shouldRegisterClickEventInsteadOfChange = function(propertyName)
    {
        return false;
    }

    this.isChecked = function(propertyName)
    {
        throw ('You must override the isChecked function if it is used.');
    }

    this.disableElement = function(propertyName)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.enableElement = function(propertyName)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.addClass = function(propertyName, cssClass)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.removeClass = function(propertyName, cssClass)
    {
        // I'm expecting that you'll use a mocking framework to see if this was called.
    }

    this.registerEventHandler = function(obj, propertyName, event, arg1, arg2, arg3)
    {
        theObj = obj;
        if (event == 'change')
        {
            var callbackArray = null;
            if (propertyChangeCallbacks[propertyName] == null)
            {
                callbackArray = new Array();
                propertyChangeCallbacks[propertyName] = callbackArray;
            }
            else
                callbackArray = propertyChangeCallbacks[propertyName];
            callbackArray.push(arg1);
        }
        else if (event == 'click')
        {
            if (propertyName == null)
                objectClickCallbacks.push(arg1);
            else
            {
                var callbackArray = null;
                if (propertyClickCallbacks[propertyName] == null)
                {
                    callbackArray = new Array();
                    propertyClickCallbacks[propertyName] = callbackArray;
                }
                else
                    callbackArray = propertyClickCallbacks[propertyName];
                callbackArray.push(arg1);
            }
        }
        else if (event == 'keypress')
        {
            var callbackArray = null;
            if (keyPressCallbacks[propertyName] == null)
            {
                callbackArray = new Array();
                keyPressCallbacks[propertyName] = callbackArray;
            }
            else
                callbackArray = keyPressCallbacks[propertyName];
            callbackArray.push(arg1);
        }
        else if (event == 'keydown')
        {
            var callbackArray = null;
            if (keyDownCallbacks[propertyName] == null)
            {
                callbackArray = new Array();
                keyDownCallbacks[propertyName] = callbackArray;
            }
            else
                callbackArray = keyDownCallbacks[propertyName];
            callbackArray.push(arg1);
        }  
        else if (event == 'blur')
        {
            var callbackArray = null;
            if (blurCallbacks[propertyName] == null)
            {
                callbackArray = new Array();
                blurCallbacks[propertyName] = callbackArray;
            }
            else
                callbackArray = blurCallbacks[propertyName];
            callbackArray.push(arg1);
        }
    }

    this.appendListItem = function(obj, listName, item)
    {
        lists[listName].push(item);
    }

    this.prependListItem = function(obj, listName, item)
    {
        lists[listName].splice(0, 0, item);
    }

    this.getAttribute = function(propertyName, attributeName)
    {
        if(attributes[propertyName] == undefined)
            return undefined;

        return attributes[propertyName][attributeName];
    }

    this.setAttribute = function(propertyName, attributeName, value)
    {
        if(attributes[propertyName] == undefined)
        {
            var list = new Object();
            attributes[propertyName] = list;
            list[attributeName] = value;
        }
        else
            attributes[propertyName][attributeName] = value;
    }

    this.toJson = function()
    {
      var json = {};
      properties = propertyRegistry.getAllProperties();
      for(var i = 0; i < properties.length; i++)
      {
        json[properties[i]] = this.getValue(properties[i]);
      }
      return json;
    }
}
